@@ -9,11 +9,11 @@ Test::Valgrind::Command - Base class for Test::Valgrind commands.
 
 =head1 VERSION
 
-Version 1.15
+Version 1.16
 
 =cut
 
-our $VERSION = '1.15';
+our $VERSION = '1.16';
 
 =head1 DESCRIPTION
 
@@ -24,6 +24,8 @@ They should also provide a default setup for generating suppressions.
 
 =cut
 
+use Test::Valgrind::Util;
+
 use base qw<Test::Valgrind::Carp>;
 
 =head1 METHODS
@@ -48,10 +50,10 @@ sub new {
 
  my %args = @_;
 
- if ($class eq __PACKAGE__ and my $cmd = delete $args{command}) {
-  $cmd =~ s/[^\w:]//g;
-  $cmd = __PACKAGE__ . "::$cmd" if $cmd !~ /::/;
-  $class->_croak("Couldn't load command $cmd: $@") unless eval "require $cmd;1";
+ my $cmd = delete $args{command};
+ if ($class eq __PACKAGE__ and defined $cmd) {
+  ($cmd, my $msg) = Test::Valgrind::Util::validate_subclass($cmd);
+  $class->_croak($msg) unless defined $cmd;
   return $cmd->new(%args);
  }
 
@@ -110,6 +112,18 @@ This method must be implemented when subclassing.
 
 sub suppressions_tag;
 
+=head2 C<check_suppressions_file>
+
+    my $supp_ok = $tvc->check_suppressions_file($file);
+
+Returns a boolean indicating whether the suppressions contained in C<$file> are compatible with the command.
+
+Defaults to true.
+
+=cut
+
+sub check_suppressions_file { 1 }
+
 =head2 C<filter>
 
     my $filtered_report = $tvc->filter($session, $report);
