@@ -9,11 +9,11 @@ Test::Valgrind::Parser - Base class for Test::Valgrind parsers.
 
 =head1 VERSION
 
-Version 1.15
+Version 1.16
 
 =cut
 
-our $VERSION = '1.15';
+our $VERSION = '1.16';
 
 =head1 DESCRIPTION
 
@@ -55,9 +55,10 @@ sub args { }
 
 =head2 C<parse>
 
-    $tvp->parse($session, $fh);
+    my $aborted = $tvp->parse($session, $fh);
 
-Parse the output of the C<valgrind> process attached to the session C<$session> received through the filehandle C<$fh>.
+Parses the output of the C<valgrind> process attached to the session C<$session> received through the filehandle C<$fh>.
+Returns true when the output indicates that C<valgrind> has aborted.
 
 This method must be implemented when subclassing.
 
